CREATE OR REPLACE PACKAGE Test1 AS

    CURSOR C_EMP IS
        select
            emp.empno,
            emp.ename,
            emp.job,
            emp.mgr,
            emp.hiredate,
            emp.sal,
            emp.comm,
            emp.deptno,
            dept.dname
        from
            emp,
            dept
        where
            emp.deptno = dept.deptno
        order by
            emp.job;
    V_EMP           C_EMP%ROWTYPE;
    Section1Num     integer;
    PriorJob        V_EMP.JOB%TYPE;
    InputBlank      CLOB;
    OutputReport    CLOB;

    PROCEDURE OpenSource;
    PROCEDURE StartReport;
    PROCEDURE CloseSource;

    PROCEDURE ExecuteReport(pMessage IN INTEGER, pW IN INTEGER, pL IN INTEGER);

END Test1;
/
CREATE OR REPLACE PACKAGE BODY Test1 AS

    PROCEDURE OpenSource IS
    BEGIN
        IF C_EMP%ISOPEN THEN
            CLOSE C_EMP;
        END IF;
        OPEN C_EMP;
        --OutputReport := NULL;
        SELECT F3 INTO OutputReport FROM BLOBTEST WHERE F1 = 30 FOR UPDATE;
        SELECT F3 INTO InputBlank FROM BLOBTEST WHERE F1 = 40;
    END;

    PROCEDURE StartReport IS
    BEGIN
        Test1.OpenSource;
        VKREP.ReportExecute('Test1.ExecuteReport', InputBlank, OutputReport);
        Test1.CloseSource;
    /*
    EXCEPTION
        WHEN OTHERS THEN
        BEGIN
            Test1.CloseSource;
            RAISE;
        END;
    */
    END;

    PROCEDURE CloseSource IS
    BEGIN
        CLOSE C_EMP;
        --UPDATE BLOBTEST SET
        --    F3 = OutputReport
        --WHERE
        --    F1 = 30;
        COMMIT;
    END;

    PROCEDURE ExecuteReport(pMessage IN INTEGER, pW IN INTEGER, pL IN INTEGER) IS
        AVG_S       NUMBER;
    BEGIN
        IF pMessage = VKREP.WM_REP_MOVE THEN
            IF pL = VKREP.MOVE_TOP THEN
                PriorJob := '';
            ELSE
                PriorJob := V_EMP.JOB;
            END IF;
            FETCH C_EMP INTO V_EMP;
            VKREP.ucCHECK := C_EMP%NOTFOUND;
            --   Not supported dynamic sections  (DBMS_SQL Limitation)
            --IF pL = VKREP.MOVE_NEXT THEN
            --    IF (pW - TRUNC(pW/2)*2) = 0 THEN
            --        --VKREP.OutSection('SECTION1');
            --        VKREP.OutSection(Section1Num);
            --    END IF;
            --END IF;
        END IF;
        IF pMessage = VKREP.WM_REP_RTF_HEADER THEN
            IF pL = VKREP.SECTION_DATA_REQUEST_BY_NAME THEN
                VKREP.RequestStr := '!';
            END IF;
        END IF;
        IF pMessage = VKREP.WM_REP_SECTION_PREPARE THEN
            Section1Num := VKREP.PrepareSection('SECTION1');
        END IF;
        IF pMessage = VKREP.WM_REP_HEADER THEN
            IF pL = VKREP.SECTION_DATA_REQUEST_BY_NUM THEN
                IF      pW = 1 THEN
                    VKREP.RequestStr := 'First parametr.';
                ELSIF   pW = 2 THEN
                    VKREP.RequestStr := 'Second parametr.';
                END IF;
            END IF;
        END IF;
        IF pMessage = VKREP.WM_REP_GROUP_HEADER THEN
            IF pL = VKREP.SECTION_CHECK THEN
                IF PriorJob <> V_EMP.JOB THEN
                    VKREP.ucCHECK := TRUE;
                END IF;
            END IF;
            IF pL = VKREP.SECTION_DATA_REQUEST_BY_NAME THEN
                VKREP.RequestStr := V_EMP.JOB;
                VKREP.RequestStr := LTRIM(RTRIM(VKREP.RequestStr));
            END IF;
            IF pL = VKREP.SECTION_DATA_REQUEST_BY_NUM THEN
                IF      pW = 1 THEN
                    SELECT AVG(SAL) INTO AVG_S FROM EMP WHERE JOB = V_EMP.JOB;
                    VKREP.RequestStr := TO_CHAR(round(AVG_S, 2));
                END IF;
            END IF;
        END IF;
        IF pMessage = VKREP.WM_REP_BODY THEN
            IF pL = VKREP.SECTION_DATA_REQUEST_BY_NUM THEN
                IF      pW = 1 THEN
                    VKREP.RequestStr := TO_CHAR(V_EMP.EMPNO);
                ELSIF   pW = 3 THEN
                    VKREP.RequestStr := V_EMP.JOB;
                ELSIF   pW = 5 THEN
                    VKREP.RequestStr := TO_CHAR(V_EMP.SAL);
                ELSIF   pW = 6 THEN
                    VKREP.RequestStr := TO_CHAR(V_EMP.DEPTNO);
                ELSIF   pW = 7 THEN
                    VKREP.RequestStr := V_EMP.DNAME;
                END IF;
                VKREP.RequestStr := LTRIM(RTRIM(VKREP.RequestStr));
            END IF;
            IF pL = VKREP.SECTION_DATA_REQUEST_BY_NAME THEN
                VKREP.RequestStr := V_EMP.ENAME;
                VKREP.RequestStr := LTRIM(RTRIM(VKREP.RequestStr));
            END IF;
        END IF;
        IF pMessage = VKREP.NEW_SECTION_START + Section1Num - 1 THEN
            --Not supported
            NULL;
        END IF;
        IF pMessage = VKREP.WM_REP_FOOTER THEN
            IF pL = VKREP.SECTION_DATA_REQUEST_BY_NUM THEN
                IF      pW = 1 THEN
                    SELECT AVG(SAL) INTO AVG_S FROM EMP;
                    VKREP.RequestStr := TO_CHAR(ROUND(AVG_S, 2));
                END IF;
            END IF;
        END IF;
    END;

END Test1;
/
